diff a/test/jdk/java/lang/reflect/code/TestBlockIndexes.java b/test/jdk/java/lang/reflect/code/TestBlockIndexes.java
--- /dev/null
+++ b/test/jdk/java/lang/reflect/code/TestBlockIndexes.java
@@ -0,0 +1,88 @@
+/*
+ * Copyright (c) 2024, Oracle and/or its affiliates. All rights reserved.
+ * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
+ *
+ * This code is free software; you can redistribute it and/or modify it
+ * under the terms of the GNU General Public License version 2 only, as
+ * published by the Free Software Foundation.
+ *
+ * This code is distributed in the hope that it will be useful, but WITHOUT
+ * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
+ * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
+ * version 2 for more details (a copy is included in the LICENSE file that
+ * accompanied this code).
+ *
+ * You should have received a copy of the GNU General Public License version
+ * 2 along with this work; if not, write to the Free Software Foundation,
+ * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
+ *
+ * Please contact Oracle, 500 Oracle Parkway, Redwood Shores, CA 94065 USA
+ * or visit www.oracle.com if you need additional information or have any
+ * questions.
+ */
+
+import org.testng.Assert;
+import org.testng.annotations.Test;
+
+import java.lang.reflect.Method;
+import java.lang.reflect.code.Block;
+import java.lang.reflect.code.Op;
+import java.lang.reflect.code.OpTransformer;
+import java.lang.reflect.code.op.CoreOp;
+import java.lang.runtime.CodeReflection;
+import java.util.Optional;
+import java.util.concurrent.atomic.AtomicBoolean;
+import java.util.stream.Stream;
+
+/*
+ * @test
+ * @run testng TestBlockIndexes
+ */
+
+public class TestBlockIndexes {
+
+    @CodeReflection
+    static int f(int[] a) {
+        int sum = 0;
+        for (int i = 0; i < a.length; i++) {
+            sum += a[i];
+        }
+        return sum;
+    }
+
+    @Test
+    public void testBlockIndexes() {
+        CoreOp.FuncOp f = getFuncOp("f");
+        f = f.transform(OpTransformer.LOWERING_TRANSFORMER);
+        assertBlockIndexes(f);
+
+        AtomicBoolean first = new AtomicBoolean(true);
+        f = f.transform((block, op) -> {
+            if (first.getAndSet(false)) {
+                // Create some blocks without predecessors
+                for (int i = 0; i < 5; i++) {
+                    Block.Builder redundant = block.block();
+                    redundant.op(CoreOp._return());
+                }
+            }
+            block.op(op);
+            return block;
+        });
+        assertBlockIndexes(f);
+    }
+
+    static void assertBlockIndexes(CoreOp.FuncOp f) {
+        for (Block b : f.body().blocks()) {
+            Assert.assertEquals(b.index(), b.parentBody().blocks().indexOf(b));
+        }
+    }
+
+    static CoreOp.FuncOp getFuncOp(String name) {
+        Optional<Method> om = Stream.of(TestBlockIndexes.class.getDeclaredMethods())
+                .filter(m -> m.getName().equals(name))
+                .findFirst();
+
+        Method m = om.get();
+        return m.getCodeModel().get();
+    }
+}
